@@ -6,22 +6,10 @@
 __author__ = "Thomas Scharrenbach (thomas@scharrenbach.net)"
 __copyright__ = "Copyright (C) 2014 Thomas Scharrenbach"
 __license__ = "Apache License v2"
-__version__ = "0.0.4"
+__version__ = "0.0.6"
 
 
 # relies on nothing
-from .classes.BandRDFGraph import BandRDFGraph
-from .classes.Raster import Raster
-from .classes.RasterRDFDataset import RasterRDFDataset
-
-from . import Raster
-from . import BandRDFGraph
-from . import RasterRDFDataset
-
-from . import gdalutils
-from . import ogcutils
-
-# relies on classes
 from .gdalrdf import create_raster_from_file
 from .gdalrdf import create_raster_from_iri
 from .gdalrdf import create_raster_from_stream
@@ -32,10 +20,10 @@ from . import create_raster_from_file
 from . import create_raster_from_iri
 from . import get_pixel_value
 
-from .classes.Raster import Raster
-from . import Raster
 from .classes.BandRDFGraph import BandRDFGraph
 from . import BandRDFGraph
+from .classes.Raster import Raster
+from . import Raster
 from .classes.RasterRDFDataset import RasterRDFDataset
 from . import RasterRDFDataset
 
@@ -44,4 +32,3 @@ __all__ = ['create_raster_from_file','create_raster_from_stream', 'create_raster
            'create_raster_from_iri', 'get_pixel_value',
            'gdalutils', 'ogcutils', 'sparql',
            'Raster', 'BandRDFGraph', 'RasterRDFDataset']
-
