@@ -8,7 +8,7 @@ This module contains mostly utilities for projecting points between different re
 __author__ = "Thomas Scharrenbach (thomas@scharrenbach.net)"
 __copyright__ = "Copyright (C) 2014 Thomas Scharrenbach"
 __license__ = "Apache License v2"
-__version__ = "0.0.4"
+__version__ = "0.0.6"
 
 import logging
 
@@ -108,7 +108,7 @@ def create_raster_base(
     :param raster_x_size: x-size of raster (default 512)
     :param raster_y_size: y-size of raster (default 256)
     :param geo_transform: affine transformation (default [0,1,0,0,0,1])
-    :param srs spatial reference system (default WGS84)
+    :param srs: spatial reference system (default WGS84)
     :return: raster without data.
     """
     driver = gdal.GetDriverByName(driver)
@@ -230,39 +230,20 @@ def geometry_geo2raster(geometry, affine_trans_inv):
     return geometry
 
 
-def clip(dataset, mask_geometry, nodata_value=-1, gdal_driver_name='MEM', ogr_driver_name='Memory'):
-    """Clip dataset with respect to a mask_geometry.
-
-    :param dataset: raster
-    :param mask_geometry: mask_geometry to be used for clip.
-    :param nodata_value: value for no data for clip.
-    :param gdal_driver_name gdal driver to use for clipping and result (default: MEM)
-    :param ogr_driver_name ogr driver to use for shape layer (default: Memory)
-    :return:
-
-    We perform the clipping by accomplishing the following steps:
-    1. Project the geometry into raster coordinates.
-    2. Compute the intersection between the projected geometry's bbox and the raster's bbox.
-        This will serve as the clipping region.
-    3. Create an ogr layer holding the mask geometry and burn it into a fresh raster, which has the dimensions of the
-        clipping region.
-    4. For each band of the raster, read the clipping region and mask it with the data from the fresh raster into which
-        we burned the mask geometry.
-    5. Write the result to a new raster with the same meta-data as the old raster. Note that this new raster holds only
-        the original data of the clipping region. The affine transformation may have been corected by the translation
-        imposed by the bounding box of the clipping region.
+def bbox_for_geometry_inside_dataset(dataset, mask_geometry):
     """
 
-    # Save the raster's affine transformation and its inverse for later computations.
-    a = affine_transformation(dataset)
-    a_inv = affine_transformation_inv(dataset)
+    :param dataset:
+    :param mask_geometry:
+    :return:
+    """
 
     # Create a copy of the mask mask_geometry and project it to the raster's SRS.
     geometry_raster = mask_geometry.Clone()
     geometry_raster.TransformTo(osr.SpatialReference(dataset.GetProjection()))
 
     # Project the mask geometry into raster coordinates.
-    geometry_geo2raster(geometry_raster, a_inv)
+    geometry_geo2raster(geometry_raster, affine_transformation_inv(dataset))
 
     # Convert the raster's bounding box to a mask_geometry in order to intersect it with the mask mask_geometry.
     bbox = points2polygon(
@@ -280,32 +261,58 @@ def clip(dataset, mask_geometry, nodata_value=-1, gdal_driver_name='MEM', ogr_dr
     win_xsize = round(env[1])
     win_ysize = round(env[3])
 
+    logging.debug('GEO_RASTER: {}'.format(geometry_raster))
+    logging.debug('ENVELOPE_GEO_RASTER: {}'.format(env))
+
     # Due to rounding errors we have to check whether the computed values fir the raster's dimensions.
     # In case they do not fit we have to correct these.
     if xoff + win_xsize > dataset.RasterXSize:
-        logging.debug('Correcting win_xsize from "{}" to "{}"'.format(win_xsize, dataset.RasterXSize - xoff))
+        logging.debug('Correcting win_xsize from "{}" to "{}"'.format(
+            win_xsize, dataset.RasterXSize - xoff))
         win_xsize = dataset.RasterXSize - xoff
     if yoff + win_ysize > dataset.RasterYSize:
-        logging.debug('Correcting win_ysize from "{}" to "{}"'.format(win_ysize, dataset.RasterYSize - yoff))
+        logging.debug('Correcting win_ysize from "{}" to "{}"'.format(
+            win_ysize, dataset.RasterYSize - yoff))
         win_ysize = dataset.RasterYSize - yoff
 
-    logging.debug('GEO_RASTER: {}'.format(geometry_raster))
-    logging.debug('ENVELOPE_GEO_RASTER: {}'.format(env))
     logging.debug('xSize: {}, ySize: {}'.format(dataset.RasterXSize, dataset.RasterYSize))
     logging.debug('xoff: {}, yoff:{}, win_x:{}, win_y:{}'.format(xoff, yoff, win_xsize, win_ysize))
 
+    return xoff, yoff, win_xsize, win_ysize
+
+
+def create_mask_dataset(dataset, mask_geometry, nodata_value=-1, gdal_driver_name='MEM', ogr_driver_name='Memory'):
+    """Create a mask raster dataset from a mask geometry.
+
+    :param dataset:
+    :param mask_geometry:
+    :param nodata_value:
+    :param gdal_driver_name:
+    :param ogr_driver_name:
+    :return:
+    """
+
+    logging.info('Started creating mask...')
+    # Compute dimensions of masked dataset.
+    xoff, yoff, win_xsize, win_ysize = bbox_for_geometry_inside_dataset(
+        dataset=dataset, mask_geometry=mask_geometry)
+
     # For creating the actual mask we will burn a shape layer into the raster.
     # To be more precise we will burn the shape into a new raster which has the dimensions of the
     # clipping region. We will use this auxiliary raster for creating the actual mask as a numpy array.
-    ogr_driver_name = 'Memory'
+    #shape_layer = create_layer_for_shape(
+    #    geometry=mask_geometry, srs=dataset.GetProjection(), ogr_driver_name=ogr_driver_name)
+
     ogr_driver = ogr.GetDriverByName(ogr_driver_name)
 
     if ogr_driver is None:
         raise Exception('Driver not available: "{}"'.format(ogr_driver_name))
 
     logging.info('Started creating layer for shape.')
-    # Create an in-memory shape dataset holding the specified mask_geometry.
-    ogr_ds = ogr_driver.CreateDataSource('/tmp/test.shp')
+    # Create an in-memory shape dataset holding the specified geometry.
+    shape_layer_file = tempfile.NamedTemporaryFile(delete=True)
+    shape_layer_file.close()
+    ogr_ds = ogr_driver.CreateDataSource(shape_layer_file.name)
     shape_layer = ogr_ds.CreateLayer(
         name='point_out', srs=osr.SpatialReference(dataset.GetProjection()),
         geom_type=ogr.wkbMultiPolygon)
@@ -322,7 +329,6 @@ def clip(dataset, mask_geometry, nodata_value=-1, gdal_driver_name='MEM', ogr_dr
     data_type = dataset.GetRasterBand(1).DataType
 
     # Create the masking dataset as a temporary file
-    logging.info('Started creating mask...')
     dst_file = tempfile.NamedTemporaryFile(delete=True)
     mask_ds = gdal_driver.Create(
         utf8_path=dst_file.name,
@@ -332,15 +338,102 @@ def clip(dataset, mask_geometry, nodata_value=-1, gdal_driver_name='MEM', ogr_dr
         eType=data_type)
     mask_ds.SetProjection(dataset.GetProjection())
 
+    # Translate affine transform according to offsets
     gt = dataset.GetGeoTransform()
-
-    upper_left = project_raster_to_geo([[xoff, yoff]], affine_trans=a)[0]
+    upper_left = project_raster_to_geo(
+        [[xoff, yoff]], affine_trans=affine_transformation(dataset))[0]
     gt_new = (upper_left[0], gt[1], gt[2], upper_left[1], gt[4], gt[5])
     mask_ds.SetGeoTransform(gt_new)
 
-    logging.info('Finished creating mask.')
+    # Burn mask_geometry to raster layer.
+    gdal.RasterizeLayer(dataset=mask_ds, bands=[1], layer=shape_layer, burn_values=[-1])
+    dst_mask_band = mask_ds.GetRasterBand(1)
+    dst_mask_band.SetNoDataValue(nodata_value)
 
+    logging.info('Finished creating mask.')
+    return mask_ds, xoff, yoff, win_xsize, win_ysize
+
+
+def clip_bands(dataset, mask_geometry, bands='ALL', nodata_value=-1, fill_value=0,
+               gdal_driver_name='MEM', ogr_driver_name='Memory'):
+    """Clip the bands of a dataset with respect to a mask_geometry.
+
+    :param dataset: raster
+    :param mask_geometry: mask_geometry to be used for clip_dataset.
+    :param bands: indices of bands to clip or 'ALL'
+    :param nodata_value: value for no data for clip_dataset.
+    :param gdal_driver_name: gdal driver to use for clipping and result (default MEM)
+    :param ogr_driver_name: ogr driver to use for shape layer (default Memory)
+    :return: list of numpy arrays, each of which holding the clipped data of the bands
+
+    We perform the clipping by accomplishing the following steps:
+
+    1. Project the geometry into raster coordinates.
+    2. Compute the intersection between the projected geometry's bbox and the raster's bbox.
+        This will serve as the clipping region.
+    3. Create an ogr layer holding the mask geometry and burn it into a fresh raster, which has the dimensions of the
+        clipping region.
+    4. For each band of the raster, read the clipping region and mask it with the data from the fresh raster into which
+        we burned the mask geometry.
+    5. Write the result to a new raster with the same meta-data as the old raster. Note that this new raster holds only
+        the original data of the clipping region. The affine transformation may have been corected by the translation
+        imposed by the bounding box of the clipping region.
+    """
+
+    mask_ds, xoff, yoff, win_xsize, win_ysize = create_mask_dataset(
+        dataset=dataset, mask_geometry=mask_geometry, nodata_value=nodata_value,
+        gdal_driver_name=gdal_driver_name, ogr_driver_name=ogr_driver_name)
+
+    dst_mask_band = mask_ds.GetRasterBand(1)
+    mask_data = dst_mask_band.ReadAsArray(
+        xoff=0, yoff=0, win_xsize=mask_ds.RasterXSize, win_ysize=mask_ds.RasterYSize)
+    mask_ds = None
+
+    band_indices = None
+    if bands == 'ALL':
+        band_indices = range(1, dataset.RasterCount+1)
+    else:
+        band_indices = bands
+
+    result = dict()
+    for band_idx in band_indices:
+        # The indices of the bands holding the original data are identical to those used for storing intermediate
+        # results.
+        src_data_band = dataset.GetRasterBand(band_idx)
+
+        src_data = src_data_band.ReadAsArray(
+            xoff=xoff, yoff=yoff, win_xsize=win_xsize, win_ysize=win_ysize)
+
+        result[band_idx] = numpy.ma.masked_array(
+            src_data,
+            numpy.logical_not(mask_data),
+            fill_value=fill_value)
+
+    return result, xoff, yoff, win_xsize, win_ysize
+
+
+def clip_dataset(dataset, mask_geometry, bands='ALL', nodata_value=-1, fill_value=0, gdal_driver_name='MEM', ogr_driver_name='Memory'):
+    """Clip dataset with respect to a mask_geometry.
+
+    :param dataset: raster
+    :param mask_geometry: mask_geometry to be used for clip_dataset.
+    :param nodata_value: value for no data for clip_dataset.
+    :param gdal_driver_name: gdal driver to use for clipping and result (default MEM)
+    :param ogr_driver_name: ogr driver to use for shape layer (default Memory)
+    :return: dataset
+    """
     logging.info('Started creating result tiff.')
+
+    gdal_driver = gdal.GetDriverByName(gdal_driver_name)
+    if gdal_driver is None:
+        raise Exception('Driver not available: "{}"'.format(gdal_driver_name))
+
+    # Compute dimensions of masked dataset.
+    xoff, yoff, win_xsize, win_ysize = bbox_for_geometry_inside_dataset(
+        dataset=dataset, mask_geometry=mask_geometry)
+
+    data_type = dataset.GetRasterBand(1).DataType
+
     # Create result dataset to temporary file.
     result_file = tempfile.NamedTemporaryFile(delete=True)
     logging.debug('Filename of result: {}'.format(result_file.name))
@@ -351,37 +444,32 @@ def clip(dataset, mask_geometry, nodata_value=-1, gdal_driver_name='MEM', ogr_dr
         bands=dataset.RasterCount,
         eType=data_type)
     result_ds.SetProjection(dataset.GetProjection())
-    result_ds.SetGeoTransform(gt_new)
+    #result_ds.SetGeoTransform(gt_new)
     result_ds.SetMetadata(dataset.GetMetadata())
     logging.info('Finished creating result tiff.')
 
-    logging.info('Started burning shape.')
-    # Burn mask_geometry to raster layer.
-    gdal.RasterizeLayer(dataset=mask_ds, bands=[1], layer=shape_layer, burn_values=[255])
-    logging.info('Finished burning shape.')
-    logging.info('Started reading burn data...')
-    dst_mask_band = mask_ds.GetRasterBand(1)
-    #dst_mask_band.SetNoDataValue(nodata_value)
-    mask_data = dst_mask_band.ReadAsArray(
-        xoff=0, yoff=0, win_xsize=mask_ds.RasterXSize, win_ysize=mask_ds.RasterYSize)
-    logging.info('Finished reading burn data.')
+    band_indices = range(1, dataset.RasterCount+1)
+    masked_band_data, xoff, yoff, win_xsize, win_ysize = clip_bands(
+        dataset=dataset, mask_geometry=mask_geometry, bands=band_indices,
+        nodata_value=nodata_value, fill_value=fill_value,
+        gdal_driver_name=gdal_driver_name, ogr_driver_name=ogr_driver_name)
 
+    band_indices = None
+    if bands == 'ALL':
+        band_indices = range(1, dataset.RasterCount+1)
+    else:
+        band_indices = bands
+
+    dst_band_idx = 0
     # Mask all bands of the original dataset
-    for band_idx in range(1, dataset.RasterCount+1):
+    for src_band_idx in band_indices:
+        dst_band_idx += 1
         # The indices of the bands holding the original data are identical to those used for storing intermediate
         # results.
-        src_data_band = dataset.GetRasterBand(band_idx)
-
-        src_data = src_data_band.ReadAsArray(
-            xoff=xoff, yoff=yoff, win_xsize=win_xsize, win_ysize=win_ysize)
-
-        result = numpy.ma.masked_array(
-            src_data,
-            numpy.logical_not(mask_data),
-            fill_value=0)
+        src_data_band = dataset.GetRasterBand(src_band_idx)
 
         # Fill bands of result dataset with masked values.
-        result_band = result_ds.GetRasterBand(band_idx)
+        result_band = result_ds.GetRasterBand(dst_band_idx)
         # TODO check which meta-data has to be copied and which not
         #result_band.SetColorTable(src_data_band.GetColorTable())
         #result_band.SetRasterColorTable(src_data_band.GetRasterColorTable())
@@ -390,11 +478,8 @@ def clip(dataset, mask_geometry, nodata_value=-1, gdal_driver_name='MEM', ogr_dr
         #result_band.SetDefaultRAT(src_data_band.GetDefaultRAT())
         #result_band.SetCategoryNames(src_data_band.GetCategoryNames())
         result_band.SetNoDataValue(nodata_value)
-        logging.info('Started masking data for band {}'.format(band_idx))
-        result_band.WriteArray(result.filled(fill_value=nodata_value))
-        logging.info('Finished masking data for band {}'.format(band_idx))
+        logging.info('Started masking data for band {}'.format(src_band_idx))
+        result_band.WriteArray(masked_band_data.get(src_band_idx).filled(fill_value=nodata_value))
+        logging.info('Finished masking data for band {}'.format(src_band_idx))
 
-    # Free resources for raster holding intermediate results.
-    mask_ds = None
-
-    return result_ds
\ No newline at end of file
+    return result_ds
